// ==================================================================
// CDGEXT32.CPP 
// ==================================================================

//
// module-level defines
//
#define INITGUID
#define USES_IID_IExchExt
#define USES_IID_IExchExtAdvancedCriteria
#define USES_IID_IExchExtAttachedFileEvents
#define USES_IID_IExchExtCommands
#define USES_IID_IExchExtMessageEvents
#define USES_IID_IExchExtPropertySheets
#define USES_IID_IExchExtSessionEvents
#define USES_IID_IExchExtUserEvents
#define USES_IID_IMessage
#define USES_PS_MAPI

//
// include files
//
#include "CDGEXT32.H"
#include <INITGUID.H>
#include <MAPIGUID.H>


//
// local declarations
//
MAPINAMEID NamedID[1]; // for new property
BOOL bSignatureOn = TRUE; // assume it's on
static HINSTANCE ghInstDLL = NULL;  // DLL handle


// -----------------------------------------------------------------
// Main Body of routines
//
// These two routines are the initial entry point for the DLL and
// the code that registers the extension DLL.
// -----------------------------------------------------------------

//
// to start things off as a DLL
//
BOOL WINAPI DllMain(
    HINSTANCE  hinstDLL,
    DWORD  fdwReason,	
    LPVOID  lpvReserved) 
{
 if (DLL_PROCESS_ATTACH == fdwReason)
 {
    ghInstDLL = hinstDLL;

 }
 return TRUE;
}

//
// register the extension
//
LPEXCHEXT CALLBACK ExchEntryPoint(void)
{
	 return new CDGExt;
}

// -------------------------------------------
// Handle the creation of the CDGExt object
// -------------------------------------------
//
// These routines establish the initial 
// extension interface.
//
// CDGExt::CDGExt() - construtor
// CDGExt::Release() - frees up resources
// CDGExt::QueryInterface() - main entry
// CDGExt::Install() - installs extensions
// --------------------------------------------

//
// constructor for the propertysheet and message event extension
//
CDGExt::CDGExt()
{
  m_cRef = 1; 
  m_pExchExtPropertySheets = new CDGExtPropSheets(this);
  m_pExchExtMessageEvents = new CDGExtMsgEvents(this);

};


//
// frees up resources when done
//
STDMETHODIMP_(ULONG) CDGExt::Release() 
{ 
 ULONG ulCount = --m_cRef;
                         
 if (!ulCount) 
 { 
  delete this;
 }

return ulCount;

}

//
// initial interface query
// for both propertysheets and message events
//
STDMETHODIMP CDGExt::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)          
{
	HRESULT hResult = S_OK;

	*ppvObj = NULL;

	if (( IID_IUnknown == riid) || ( IID_IExchExt == riid) )
	{
	    *ppvObj = (LPUNKNOWN)this;
	}
	else if (IID_IExchExtPropertySheets == riid) // for property sheets?
	{
	 // ignore send/read contexts (for property sheet)
	    if ( (m_context == EECONTEXT_SENDNOTEMESSAGE)   ||
	         (m_context == EECONTEXT_SENDPOSTMESSAGE)   ||
	         (m_context == EECONTEXT_SENDRESENDMESSAGE) ||
	         (m_context == EECONTEXT_READNOTEMESSAGE)   ||
	         (m_context == EECONTEXT_READPOSTMESSAGE)   ||
	         (m_context == EECONTEXT_READREPORTMESSAGE) )
            return E_NOINTERFACE;

	 // otherwise return the interface
	    *ppvObj = (LPUNKNOWN) m_pExchExtPropertySheets;
	}
	else if (IID_IExchExtMessageEvents == riid) // for message events?
	{
		*ppvObj = (LPUNKNOWN) m_pExchExtMessageEvents;
	}
	else
		hResult = E_NOINTERFACE;

	if (NULL != *ppvObj)
		((LPUNKNOWN)*ppvObj)->AddRef();

	return hResult;
}


//
// actually installs the extension
//
STDMETHODIMP CDGExt::Install(LPEXCHEXTCALLBACK peecb, ULONG eecontext, ULONG ulFlags)
{
	ULONG ulBuildVersion;
	HRESULT hr;

	m_context = eecontext;
	   
    // compare versions 
	peecb->GetVersion(&ulBuildVersion, EECBGV_GETBUILDVERSION);
    if (EECBGV_BUILDVERSION_MAJOR != (ulBuildVersion & 
                                      EECBGV_BUILDVERSION_MAJOR_MASK))
	    return S_FALSE;  // oops!

	switch (eecontext)
	{
	 case EECONTEXT_PROPERTYSHEETS:
	 case EECONTEXT_SENDNOTEMESSAGE:
	 case EECONTEXT_SENDPOSTMESSAGE:
	 case EECONTEXT_SENDRESENDMESSAGE:
	 case EECONTEXT_READNOTEMESSAGE:
	 case EECONTEXT_READPOSTMESSAGE:
	 case EECONTEXT_READREPORTMESSAGE:
	 	hr = S_OK;
		break;

	 default:
	 	hr = S_FALSE;
		break;
	}


    return hr;

}
//
// end of CDGExt Object 
// ----------------------------------------------------------------


// -----------------------------------------------------------------
// Handle creation of Messge Event object
// -----------------------------------------------------------------
//
// Routines for the Message Event extension
//
// CDGExtMsgEvents::QueryInterface() - [not used]
// CDGExtMsgEvents::OnRead() - at start of read msg
// CDGExtMsgEvents::OnReadComplete() - at end of read
// CDGExtMsgEvents::OnWrite() - ad start of write msg
// CDGExtMsgEvents::OnWriteComplete() - at end of write
// CDGExtMsgEvents::OnSubmit() - at start of msg submit
// CDGExtMsgEvents::OnSubmitComplete() - end of submit
// CDGExtMsgEvents::OnCheckNames() - start of resolve
// CDGExtMsgEvents::OnCheckNamesComplete() - resolve end
// ------------------------------------------------------------------

//
// sample queryinterface for message events
// not used now
//
STDMETHODIMP CDGExtMsgEvents::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)          
{	

	*ppvObj = NULL;
    if (riid == IID_IExchExtMessageEvents)
    {
        *ppvObj = (LPVOID)this;
        AddRef(); // using one more!
        return S_OK;
    }
	if (riid == IID_IUnknown)
	{
		*ppvObj = (LPVOID)m_pExchExt;  // return parent interface
		m_pExchExt->AddRef();
		return S_OK;
	}
    return E_NOINTERFACE;

}


//
// fires when user attempts to Read a message
//
HRESULT CDGExtMsgEvents::OnRead(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;
  LPMESSAGE pMsg;
  LPMDB pMDB;
  ULONG ulCheckSum;
  LPSPropTagArray pNamedPropTags;
  LPSPropValue pPropValues;
  ULONG ulcValues;
  HWND hWnd;
  HCURSOR hOldCursor;

  m_hrOnReadComplete = S_FALSE; // assume all will go fine

  pMsg = NULL;
  pMDB = NULL;
  pPropValues = NULL;
  pNamedPropTags = NULL;

  // turned OFF?
  if (!bSignatureOn)
  {
  	goto error_return;
  }

  // get the message
  hr = lpeecb->GetObject(&pMDB, (LPMAPIPROP *)&pMsg);
  if (FAILED(hr))
  {
	goto error_return;
  }

  // save current state of window and cursor
  lpeecb->GetWindow(&hWnd);
  hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));

  // check to see if MsgChecksum exists
  NamedID[0].lpguid = (LPGUID)&IID_IMessage;
  NamedID[0].ulKind = MNID_STRING;
  NamedID[0].Kind.lpwstrName = L"MsgChecksum";

  hr = pMsg->GetIDsFromNames(1, (LPMAPINAMEID *)&NamedID, 0, &pNamedPropTags);
  if (FAILED(hr)) // maybe not there first time
  {
  	goto error_return;
  }

  hr = pMsg->GetProps(pNamedPropTags, 0, &ulcValues, &pPropValues);
  if (FAILED(hr))    // must be some other error
  {
    goto error_return;
  }

  if (hr == MAPI_W_ERRORS_RETURNED) // not signed
  {
    goto error_return;
  }

  // must be signed, check the signature
  hr = CheckMsgSignature(pMsg, &ulCheckSum);
  if (FAILED(hr))
  {
    ErrMsgBox(hWnd, hr, "OnRead", 
		"An error occured while calculating\n"
		"the message signature.");
  	goto error_return;
  }

  //
  // got a value back
  //
  if (pPropValues[0].Value.ul == ulCheckSum)
  {
      MessageBox(hWnd, 
		  "Signed Message Verified.",
		  "CDG Message Signing Extension", 
		  MB_OK);
  }
  else
  {
   int nRet = MessageBox(hWnd, 
	   "Signed Message was altered.\n"
	   "Do you wish to view the message anyway?",
	   "CDG MEssage Signing Extension", 
	   MB_YESNO);
   
   if (nRet == IDNO)
   // tell OnReadComplete to not display the message
      m_hrOnReadComplete = MAPI_E_CALL_FAILED;
  }

  error_return:

  hr = S_FALSE;

  //
  // free up resources
  //
  if (pMDB != NULL)
  	pMDB->Release();

  if (pMsg != NULL)
  	pMsg->Release();

  if (pNamedPropTags != NULL)
  	MAPIFreeBuffer(pNamedPropTags);

  if (pPropValues != NULL)
  	MAPIFreeBuffer(pPropValues);

  SetCursor(hOldCursor);

  return hr;
}

//
// fires after the read is complete
//
HRESULT CDGExtMsgEvents::OnReadComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{

  return m_hrOnReadComplete;
}

//
// fires when user starts to write message
//
HRESULT CDGExtMsgEvents::OnWrite(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;

  hr = S_FALSE; 

  return hr;
}

//
// fires after message has been written
//
HRESULT CDGExtMsgEvents::OnWriteComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{
  HRESULT hr;
  LPMESSAGE pMsg;
  LPMDB pMDB;
  ULONG ulCheckSum;
  SPropValue pChksumProp[1];
  LPSPropTagArray pNamedPropTags;
  LPSPropProblemArray pPropProblems;
  HWND hWnd;
  HCURSOR hOldCursor;

  // turned OFF?
  if (!bSignatureOn)
  {
  	return S_FALSE; 
  }

  // just saving or really sending?
  if (!m_bInSubmitState)
  {
  	return S_FALSE;
  }

  pMsg = NULL;
  pMDB = NULL;
  pPropProblems	= NULL;
  pNamedPropTags = NULL;

  hOldCursor = SetCursor(LoadCursor(NULL, IDC_WAIT));


  hr = lpeecb->GetObject(&pMDB, (LPMAPIPROP *)&pMsg);
  if (FAILED(hr))
  {
	goto error_return;
  }

  hr = CheckMsgSignature(pMsg, &ulCheckSum);
  if (FAILED(hr))
  {
    lpeecb->GetWindow(&hWnd);

    ErrMsgBox(hWnd, hr, "OnWriteComplete", 
		"An error occured while calculating\n"
		"the message signature.");
	goto error_return;
  }

  // use a named property for the message body checksum
  NamedID[0].lpguid = (LPGUID)&IID_IMessage;
  NamedID[0].ulKind = MNID_STRING;
  NamedID[0].Kind.lpwstrName = L"MsgChecksum";

  hr = pMsg->GetIDsFromNames(1, (LPMAPINAMEID *)&NamedID, MAPI_CREATE, &pNamedPropTags);
  if (FAILED(hr))
  {
   goto error_return;
  }

  pChksumProp[0].ulPropTag = PROP_TAG(PT_LONG, HIWORD(pNamedPropTags->aulPropTag[0]));
  pChksumProp[0].dwAlignPad = 0L;
  pChksumProp[0].Value.ul = ulCheckSum;

  hr = pMsg->SetProps(1, pChksumProp, &pPropProblems);
  if (FAILED(hr))
  {
   goto error_return;
  }

  
  // all fine, keep going
  hr = S_FALSE; 

  error_return:


  //
  // free up resources
  //
  if (pMDB != NULL) 
    pMDB->Release();

  if (pMsg != NULL)
  	pMsg->Release();

  if (pNamedPropTags != NULL)
  	MAPIFreeBuffer(pNamedPropTags);

  if (pPropProblems != NULL)
  	MAPIFreeBuffer(pPropProblems);

  SetCursor(hOldCursor);

  return hr;
}


//
// fires when user attempts to submit a msg to the outbox
//
HRESULT CDGExtMsgEvents::OnSubmit(LPEXCHEXTCALLBACK lpeecb)
{
  HRESULT hr;

  hr = S_FALSE;   
  m_bInSubmitState = TRUE;  // submit is called

  return hr;
}

//
// fires after message was submitted
//
VOID CDGExtMsgEvents::OnSubmitComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{

  m_bInSubmitState = FALSE;  // out of submit state

}

//
// fires when user attempts to resolve addreses
//
HRESULT CDGExtMsgEvents::OnCheckNames(LPEXCHEXTCALLBACK lpeecb)
{
  return S_FALSE; // not used
}

//
// fires after resolve is done
//
HRESULT CDGExtMsgEvents::OnCheckNamesComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags)
{
  return S_FALSE; // not used
}

//
// end of CDGExtMsgEvents object
// ------------------------------------------------------------------


// ----------------------------------------------------------------
// Handle creation CDGExtPropSheets object
// ----------------------------------------------------------------
//
// Routines for the propertysheet extension
//
// CDGExtPropSheets::QueryInterface() - [not used]
// CDGExtPropSheets::GetMaxPageCount() - count pages
// CDGExtPropSheets::GetPages() - add a new page
// CDGExtPropSheets::FreePages() - free up resources
// ---------------------------------------------------------------



//
// sample queryinterface for propertysheets
// not used right now
//
STDMETHODIMP CDGExtPropSheets::QueryInterface(REFIID riid, LPVOID FAR * ppvObj)          
{	

	*ppvObj = NULL;
    if (riid == IID_IExchExtPropertySheets)
    {
        *ppvObj = (LPVOID)this;
        // Increase usage count of this object
        AddRef();
        return S_OK;
    }
	if (riid == IID_IUnknown)
	{
		*ppvObj = (LPVOID)m_pExchExt;  // return parent interface
		m_pExchExt->AddRef();
		return S_OK;
	}

    return E_NOINTERFACE;

}


// 
// get property page count
//
ULONG CDGExtPropSheets::GetMaxPageCount(ULONG ulFlags)          
{
 ULONG ulNumExtSheets;

    switch (ulFlags)
	{
	 // ignore these objects.
	 case EEPS_FOLDER:
	 case EEPS_STORE:
	 case EEPS_MESSAGE:
	 	ulNumExtSheets = 0;
	 	break;

     // add a tab to the Tools|Options page		
	 case EEPS_TOOLSOPTIONS:
		ulNumExtSheets = 1;	
		break;

	 default:
	 	ulNumExtSheets = 0;
		break;
	}

	return ulNumExtSheets;  
}


//
// to add a new page
//
STDMETHODIMP CDGExtPropSheets::GetPages(LPEXCHEXTCALLBACK peecb,
                      ULONG ulFlags, LPPROPSHEETPAGE ppsp, ULONG FAR * pcpsp)
{
 LPMDB pMDB = NULL;
 LPMESSAGE pItem = NULL;

    *pcpsp = 0;


	// members for the new property page
	ppsp[0].dwSize = sizeof (PROPSHEETPAGE);
    ppsp[0].dwFlags = PSP_DEFAULT | PSP_HASHELP;
	ppsp[0].hInstance = ghInstDLL;
	ppsp[0].pszTemplate = MAKEINTRESOURCE(IDD_SIGNATURE);
	ppsp[0].hIcon = NULL;     // not used
	ppsp[0].pszTitle = NULL;  // not used
	ppsp[0].pfnDlgProc = (DLGPROC)MsgSigningDlgProc;
	ppsp[0].lParam = 0;     
	ppsp[0].pfnCallback = NULL;
	ppsp[0].pcRefParent = NULL; // not used

	*pcpsp = 1;

	return S_OK;
}


//
// free up any resources
//
VOID CDGExtPropSheets::FreePages(LPPROPSHEETPAGE ppsp, ULONG ulFlags, ULONG cpsp)          
{
   // not used
}
//
// end of CDGExtPropSheets object
// -----------------------------------------------------------------



// ------------------------------------------------------------------
// Local Helper routines
// ------------------------------------------------------------------
//
// ErrMsgBox() - used to handle error msgs
// CheckMsgSignature() - calculates checksum
// ------------------------------------------------------------------

// 
// ErrMsgBox()
//
//    Params:
//      hWnd       - parent window
//      hr         - HRESULT value (0 to suppress)
//      szFunction - function name in which the error occurred (NULL to suppress)
//      szMessage  - error message (required)
//
void ErrMsgBox(HWND hWnd, HRESULT hr, LPSTR szFunction, LPSTR szMessage)
{
 static char szError[256];

 if (szMessage == NULL)
 {
	MessageBox(hWnd, 
		 "An unknown error occured in\nextension",
	     "CDG Message Signing Extension", MB_ICONEXCLAMATION | MB_OK);
	return;
 }

 if ((hr == 0) && (szFunction == NULL))
 {
    MessageBox(hWnd, szMessage, "CDG Message Signing Extension Error", MB_ICONEXCLAMATION | MB_OK);
	return;
 }


 if (szFunction != NULL) 
 {
  wsprintf(szError, "Error %08X in %s\n%s", hr, szFunction, szMessage);
  MessageBox(hWnd, szError, "CDG Message Signing Extension Error", MB_ICONEXCLAMATION | MB_OK);
 }

}



//
// CheckMsgSignature(pMsg, &ulCheckSum)()
//
//    Params:
//       pMsg         - points to message object
//       *pulCheckSum - points to checksum
//
HRESULT CheckMsgSignature(LPMESSAGE pMsg, ULONG *pulCheckSum)
{
 HRESULT hr;
 LPSTREAM pStreamBody;
 ULONG ulValue;
 ULONG ulRead;
 LARGE_INTEGER LgInt;
 ULARGE_INTEGER uLgInt;

 //
 // make sure you have a valid msg body
 //
 if ( (pMsg == NULL) ||(pulCheckSum == NULL) )
 {
	  hr = MAPI_E_INVALID_PARAMETER;
      goto error_return;
 }

 //
 // access the message body
 //
 pStreamBody = NULL;
 hr = pMsg->OpenProperty(PR_BODY, &IID_IStream, STGM_DIRECT | STGM_READ, 0, (LPUNKNOWN *) &pStreamBody);
 if (FAILED(hr))
 {
 	goto error_return;
 }

 //
 // point to starting position
 //
 LgInt.LowPart = 0;
 LgInt.HighPart = 0;
 pStreamBody->Seek(LgInt, STREAM_SEEK_SET, &uLgInt);  
 
 //
 // add up ascii values
 //
 (*pulCheckSum) = 0;
 ulValue = 0;
 while ( (S_OK == (hr = pStreamBody->Read((LPVOID)&ulValue, 4, &ulRead)))  &&
         (ulRead > 0) )
 {
    (*pulCheckSum) += ulValue;
    ulValue = 0;
 }      


 //
 // clean up
 //
 error_return:

 if (pStreamBody != NULL)
 	pStreamBody->Release();

 return hr;

}
//
// End of Helper routines
// ------------------------------------------------------------------
