// ================================================================
//  CDGPRP32.CPP
// ================================================================
//


#include "CDGEXT32.H"


// ----------------------------------------------------------------
// MsgSigningDlgProc
//
//    Params:
//      hDlg   - handle to modeless dialog, the property page
//      uMsg   - message
//      wParam - wParam of wndproc
//      lParam - lParam of wndproc, points to NMHDR for notifications
//
// Handles events for the custom property page
// ----------------------------------------------------------------

BOOL CALLBACK MsgSigningDlgProc(HWND hDlg, UINT uMsg, 
		WPARAM wParam, LPARAM lParam)
{
 BOOL bMsgResult;
 static HBRUSH hBrush;
 static COLORREF GrayColor;
 static LPNMHDR pnmhdr;
 static HWND hWndPage;

 switch (uMsg)
 {

  case WM_INITDIALOG:
  {
   LOGBRUSH lb;

    GrayColor = (COLORREF)GetSysColor(COLOR_BTNFACE);

	memset(&lb, 0, sizeof(LOGBRUSH));
	lb.lbStyle = BS_SOLID;
	lb.lbColor = GrayColor;
	hBrush = CreateBrushIndirect(&lb);

    return TRUE;
  }
  break;

  case WM_CTLCOLORDLG:
  case WM_CTLCOLORBTN:
  case WM_CTLCOLORSTATIC:
  
  	if (hBrush != NULL)
	{
	  SetBkColor((HDC)wParam, GrayColor);

	  return (BOOL)hBrush;
	}

  break;

  case WM_DESTROY:
  {
   if (hBrush != NULL)
   		DeleteObject(hBrush);
    

   return TRUE;
  }
  
  case WM_COMMAND:
  {
   if (LOWORD(wParam) == IDC_ENABLESGN)
   {
      SendMessage(GetParent(hDlg), PSM_CHANGED, (WPARAM)hDlg, 0L);
	  bSignatureOn = SendDlgItemMessage(hDlg, IDC_ENABLESGN, BM_GETCHECK, 0, 0L);
   }
  }
  break;

  case WM_NOTIFY:
  {
   pnmhdr = ((LPNMHDR) lParam);

   switch ( pnmhdr->code)
   {
	case PSN_KILLACTIVE:
		bMsgResult = FALSE;  // allow this page to receive PSN_APPLY
		break;

	case PSN_SETACTIVE:

	    // initialize controls
    	if (bSignatureOn)
		    SendDlgItemMessage(hDlg, IDC_ENABLESGN, BM_SETCHECK, 1, 0L);
        else
        	SendDlgItemMessage(hDlg, IDC_ENABLESGN, BM_SETCHECK, 0, 0L);

		hWndPage = pnmhdr->hwndFrom;   // to be used in WM_COMMAND
    		 
		bMsgResult = FALSE; 
		break;

	case PSN_APPLY:

        // get user input
		bSignatureOn = SendDlgItemMessage(hDlg, IDC_ENABLESGN, BM_GETCHECK, 0, 0L);

	    bMsgResult = PSNRET_NOERROR;
		break;

	case PSN_HELP:												
		MessageBox( pnmhdr->hwndFrom,
					"CDG Message Signing Extension\n"
					"(c)1996 MCA/SAMS Publishing",
					"About",
					MB_OK);
		bMsgResult = TRUE;  
		break;


	default:
		bMsgResult = FALSE;
		break;
   }  // switch

  	SetWindowLong( hDlg, DWL_MSGRESULT, bMsgResult);
  	break;
  
  }	  // case WM_NOTIFY

  default:
  	bMsgResult = FALSE;
	break;

 }  // switch


 return bMsgResult;
}
