Attribute VB_Name = "OLEMAPI"
Option Explicit

'
' OLE message objects
Public objSession As Object
Public objMsgColl As Object
Public objMsg As Object
Public objRecipColl As Object
Public objRecip As Object
Public objAttachColl As Object
Public objAttach As Object
Public objAddrEntry As Object
Public objUserEntry As Object
Public objFolderColl As Object
Public objFolder As Object
Public objInfoStoreColl As Object
Public objInfoStore As Object
Public gnIndentlevel
Public cStoreID As String

'
' UDT for store/folder pairs
Type FolderType
    Name As String
    FolderID As String
    StoreID As String
End Type
'
Public FolderRec() As FolderType    ' members
Public iFldrCnt As Integer          ' pointer

'
' UDT for message/conversation pairs
Type MsgType
    Subject As String
    Topic As String
    ConvIndex As String
    MsgID As String
End Type
'
Public MsgRec() As MsgType          ' members
Public iMsgCnt As Integer           ' pointer

'
' type for creating Exchange-compliant timestamp
Type GUID
    guid1 As Long
    guid2 As Long
    guid3 As Long
    guid4 As Long
End Type
'
Declare Function CoCreateGuid Lib "OLE32.DLL" (pGuid As GUID) As Long
Public Const S_OK = 0

Public Sub OLEMAPIStart()
    '
    ' start an OLE MAPI session
    '
    Set objSession = CreateObject("MAPI.Session")
    objSession.Logon
    '
End Sub

Public Sub OLEMAPIEnd()
    '
    On Error Resume Next
    objSession.Logoff ' end mapi session
    '
End Sub

Public Sub OLEMAPIPostMsg(cFolderName As String, cTopic As String, cSubject As String, cBody As String)
    '
    ' post a message to the folder (starts a new thread)
    '
    ' -----------------------------
    ' Inputs:
    '   cFolderName     - name of target folder
    '   cTopic          - general discussion topic
    '   cSubject        - user's subject line
    '   cBody           - user's body text
    ' -----------------------------
    '
    Dim uFolder As FolderType
    '
    ' get folder structure
    uFolder = GetFolderRec(cFolderName) ' get the structure
    If uFolder.FolderID = "" Then
        MsgBox "Unable to Locate Folder!", vbExclamation, cFolderName
        Exit Sub
    End If
    '
    ' open folder, store
    Set objFolder = objSession.GetFolder(uFolder.FolderID, uFolder.StoreID)
    If objFolder Is Nothing Then
        MsgBox "Unable to open folder!", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' create new message
    Set objMsg = objFolder.Messages.Add
    If objMsg Is Nothing Then
        MsgBox "Unable to create new message in folder!", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' fix up subject/topic
    If cTopic = "" And cSubject = "" Then
        cTopic = "Thread #" & Format(Now(), "YYMMDDHHmm")
    End If
    '
    If cTopic = "" And cSubject <> "" Then
        cTopic = cSubject
    End If
    '
    If cSubject = "" And cTopic <> "" Then
        cSubject = cTopic
    End If
    '
    ' now compose the message
    With objMsg
        '
        ' set user-supplied info
        .Subject = cSubject
        .Text = cBody
        '
        ' set stock properties
        .Type = "IPM.Discuss"
        .ConversationTopic = cTopic
        .ConversationIndex = MakeTimeStamp
        .TimeSent = Now()
        .TimeReceived = .TimeSent
        .Submitted = True
        .Unread = True
        .Sent = True
        '
        .Update ' force msg into the folder
    End With
    '
    objFolder.Update ' update the folder object
    '
End Sub

Public Sub OLEMAPIGetMsgs(cFolderName As String, Optional ctrl As Variant)
    '
    ' get all the discussion messages
    '
    Dim uFolder As FolderType
    ReDim MsgRec(0)
    Dim bLoadCtrl As Boolean
    '
    Set objFolder = Nothing
    Set objMsgColl = Nothing
    Set objMsg = Nothing
    '
    iMsgCnt = 0
    uFolder = GetFolderRec(cFolderName)
    '
    ' check for optional control
    If IsMissing(ctrl) Or IsNull(ctrl) Then
        ctrl = Null
        bLoadCtrl = False
    Else
        bLoadCtrl = True
        ctrl.Clear
    End If
    '
    ' now walk through folder to find discussion msgs
    If uFolder.Name = "" Then
        MsgBox "Unable to locate folder!", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' open store, folder
    Set objFolder = objSession.GetFolder(uFolder.FolderID, uFolder.StoreID)
    If objFolder Is Nothing Then
        MsgBox "Unable to open folder!", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' get message collection
    Set objMsgColl = objFolder.Messages
    If objMsgColl Is Nothing Then
        MsgBox "No messages in folder", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' ok, get first message
    On Error Resume Next
    Set objMsg = objMsgColl.GetFirst
    '
    ' now walk through folder to get all discussion msgs
    Do Until objMsg Is Nothing
        If objMsg.Type = "IPM.Discuss" Then
             iMsgCnt = iMsgCnt + 1
             ReDim Preserve MsgRec(iMsgCnt)
             MsgRec(iMsgCnt).Subject = objMsg.Subject
             MsgRec(iMsgCnt).Topic = objMsg.ConversationTopic
             MsgRec(iMsgCnt).ConvIndex = objMsg.ConversationIndex
             MsgRec(iMsgCnt).MsgID = objMsg.ID
             '
             ' optionally load control
             If bLoadCtrl = True Then
                 ctrl.AddItem objMsg.Subject
             End If
        End If
        Set objMsg = objMsgColl.GetNext
    Loop
    '
    Beep ' tell 'em you're done!
    '
End Sub

Public Function MakeTimeStamp() As String
    '
    ' create Exchange-type time stamp
    '
    Dim lResult As Long
    Dim lGuid As GUID
    '
    On Error GoTo LocalErr
    '
    lResult = CoCreateGuid(lGuid)
    If lResult = S_OK Then
        MakeTimeStamp = Hex$(lGuid.guid1) & Hex$(lGuid.guid2)
    Else
        MakeTimeStamp = "00000000"   ' zeroes
    End If
    Exit Function
    
LocalErr:
    MsgBox "Error " & Str(Err) & ": " & Error$(Err)
    MakeTimeStamp = "00000000"
    Exit Function
    '
End Function

Public Sub OLEMAPIReply(FolderName As String, iCount As Integer, cSubject As String, cBody As String)
    '
    ' post a message reply (continues existing thread)
    '
    ' -----------------------------------
    ' Inputs:
    '   FolderName  - string name of target folder
    '   iCount      - index into msg array (points to source msg)
    '   cSubject    - user's subject line (if null uses "RE:" & source subject)
    '   cBody       - user's msg body
    ' -----------------------------------
    '
    Dim uFolder As FolderType
    Dim objSourceMsg As Object
    '
    ' check msg pointer
    If iCount < 0 Then
        MsgBox "No Message Selected", vbExclamation, "OLEMAPIReply"
        Exit Sub
    End If
    '
    ' get folder for posting
    uFolder = GetFolderRec(FolderName)
    If uFolder.FolderID = "" Then
        MsgBox "Unable to locate folder!", vbExclamation, FolderName
        Exit Sub
    End If
    '
    ' get source message
    Set objSourceMsg = objSession.GetMessage(MsgRec(iCount + 1).MsgID, uFolder.StoreID)
    If objSourceMsg Is Nothing Then
        MsgBox "Unable to Load selected Message!", vbExclamation, MsgRec(iCount + 1).Subject
        Exit Sub
    End If
    '
    ' open target store, folder
    Set objFolder = objSession.GetFolder(uFolder.FolderID, uFolder.StoreID)
    If objFolder Is Nothing Then
        MsgBox "Unable to open target folder", vbExclamation, uFolder.Name
        Exit Sub
    End If
    '
    ' create a new blank message object
    Set objMsg = objFolder.Messages.Add
    If objMsg Is Nothing Then
        MsgBox "Unable to add message to folder", vbExclamation, objFolder.Name
        Exit Sub
    End If
    '
    ' fix up target subject, if needed
    If cSubject = "" Then
        cSubject = "RE: " & objSourceMsg.Subject
    End If
    '
    ' now compose reply msg
    With objMsg
        '
        ' user properties
        .Subject = cSubject
        .Text = cBody
        '
        ' stock properties
        .Type = "IPM.Discuss"
        .ConversationTopic = objSourceMsg.ConversationTopic
        .ConversationIndex = objSourceMsg.ConversationIndex & MakeTimeStamp
        .TimeSent = Now()
        .TimeReceived = .TimeSent
        .Submitted = True
        .Unread = True
        .Sent = True
        '
        .Update ' force msg into folder
    End With
    '
    objFolder.Update ' update folder object
    '
End Sub

Public Sub CollectFolders(Optional ctrl As Variant)
    '
    ' build folder tree
    '
    Dim objRootFolder As Object
    Dim lobjFolders As Object
    Dim nIter As Integer
    Dim nStoreCount
    Dim bLoadCtrl As Boolean
    '
    ' loading optional display?
    If IsMissing(ctrl) Or IsNull(ctrl) Then
        ctrl = Null
        bLoadCtrl = False
    Else
        bLoadCtrl = True
    End If
    '
    gnIndentlevel = 0
    cStoreID = ""
    '
    ' get collection of information stores
    Set objInfoStoreColl = objSession.InfoStores
    nStoreCount = objInfoStoreColl.Count
    '
    ' walk through all stores to get folders
    For nIter = 1 To nStoreCount
        iFldrCnt = iFldrCnt + 1
        ReDim Preserve FolderRec(iFldrCnt)
        FolderRec(iFldrCnt).StoreID = objInfoStoreColl.Item(nIter).ID
        FolderRec(iFldrCnt).FolderID = "" ' no folder for a store!
        FolderRec(iFldrCnt).Name = objInfoStoreColl.Item(nIter).Name
        cStoreID = FolderRec(iFldrCnt).StoreID ' hold this for the other folders
        '
        ' add to display control
        If bLoadCtrl = True Then
            ctrl.AddItem FolderRec(iFldrCnt).Name ' add to display
        End If
        '
        ' point to top of store and start loading
        Set objRootFolder = objInfoStoreColl.Item(nIter).RootFolder
        Set objFolderColl = objRootFolder.Folders
        Set lobjFolders = objFolderColl
        LoadFolder lobjFolders, ctrl
        gnIndentlevel = 0
    Next nIter
    '
End Sub

Sub LoadFolder(aFolders As Object, Optional ctrl As Variant)
    '
    ' look for folders in the collection
    '
    Dim mobjFolder As Object
    Dim mFolderCol As Object
    Dim bLoadCtrl As Boolean
    '
    gnIndentlevel = gnIndentlevel + 1
    '
    If IsMissing(ctrl) Or IsNull(ctrl) Then
        bLoadCtrl = False
    Else
        bLoadCtrl = True
    End If
    '
    Set mobjFolder = aFolders.GetFirst
    While Not mobjFolder Is Nothing
        iFldrCnt = iFldrCnt + 1
        ReDim Preserve FolderRec(iFldrCnt)
        '
        FolderRec(iFldrCnt).StoreID = cStoreID
        FolderRec(iFldrCnt).FolderID = mobjFolder.ID
        FolderRec(iFldrCnt).Name = mobjFolder.Name
        '
        ' optionally load screen display
        If bLoadCtrl = True Then
            ctrl.AddItem Space(gnIndentlevel * 5) & FolderRec(iFldrCnt).Name
        End If
        '
        ' look for nested folders
        Set mFolderCol = mobjFolder.Folders
        LoadFolder mFolderCol, ctrl
        '
        ' done with nested folders
        gnIndentlevel = gnIndentlevel - 1
        Set mobjFolder = aFolders.GetNext
    Wend
    '
End Sub

Public Function GetFolderRec(cFolderName As String) As FolderType
    '
    ' take name, return structure
    '
    Dim x As Integer
    Dim y As Integer
    '
    ' start w/ a blank return
    GetFolderRec.StoreID = ""
    GetFolderRec.FolderID = ""
    GetFolderRec.Name = ""
    '
    y = UBound(FolderRec) ' get total recs in array
    '
    ' walk through array
    For x = 0 To y - 1
        If Trim(UCase(FolderRec(x).Name)) = Trim(UCase(cFolderName)) Then
            GetFolderRec = FolderRec(x)
            Exit Function
        End If
    Next x
    '
End Function


Public Function GetMsgRec(iPointer As Integer) As MsgType
    '
    ' accept pointer, return strucutre
    '
    
    '
    ' start w blank records
    GetMsgRec.MsgID = ""
    GetMsgRec.ConvIndex = ""
    GetMsgRec.Subject = ""
    GetMsgRec.Topic = ""
    
    '
    ' now try to find it
    If iPointer < 0 Or iPointer > UBound(MsgRec) Then
        MsgBox "Invalid Message pointer!", vbExclamation, "GetMsgRec"
        Exit Function
    Else
        GetMsgRec = MsgRec(iPointer + 1)
    End If
    '
End Function
