// ========================================================================
//  CDGEXT32.H
// ========================================================================
#ifndef __CDGEXT32_H__
#define __CDGEXT32_H__

//
// include files
//
#include <WINDOWS.H> 
#include <COMMCTRL.H>
#include <MAPIX.H>
#include <MAPIUTIL.H>
#include <MAPIFORM.H>
#include <EXCHEXT.H>

#include "RESOURCE.H"

//
// function prototypes
//
extern "C"
{
 LPEXCHEXT CALLBACK ExchEntryPoint(void);
}

void ErrMsgBox(HWND hWnd, 
			   HRESULT hr, 
			   PSTR szFunction, 
			   LPSTR szMessage);

HRESULT CheckMsgSignature(LPMESSAGE pMsg,
						  ULONG *pulCheckSum);

BOOL CALLBACK MsgSigningDlgProc(HWND hDlg, 
   	                            UINT uMsg, 
								WPARAM wParam, 
								LPARAM lParam);

// global declarations
extern BOOL bSignatureOn;
extern HINSTANCE ghInstDLL;

//
// class declarations
//
class CDGExt;
class CDGExtPropSheets;
class CDGExtMsgEvents;

//
// overall exchange extension class
//
class CDGExt : public IExchExt
{
 
 public:
    CDGExt(); 
    STDMETHODIMP QueryInterface(REFIID riid, LPVOID * ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef() { ++m_cRef; return m_cRef; };
    STDMETHODIMP_(ULONG) Release(); 
    STDMETHODIMP Install (LPEXCHEXTCALLBACK pmecb, ULONG mecontext, ULONG ulFlags);
    
 private:
    ULONG m_cRef;
    UINT  m_context;
	CDGExtPropSheets * m_pExchExtPropertySheets;
	CDGExtMsgEvents * m_pExchExtMessageEvents;

};

//
// property sheet extension class
//
class CDGExtPropSheets : public IExchExtPropertySheets
{
 public:
    CDGExtPropSheets (LPUNKNOWN pParentInterface) { 
    m_pExchExt = pParentInterface;
    m_cRef = 0; 
    };

    
    STDMETHODIMP QueryInterface(REFIID   riid, LPVOID * ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef() { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release() 
	      { ULONG ulCount = --m_cRef;
	        if (!ulCount) { delete this; }
           return ulCount;};

	STDMETHODIMP_ (ULONG) GetMaxPageCount(ULONG ulFlags);          
	STDMETHODIMP  GetPages(LPEXCHEXTCALLBACK peecb, ULONG ulFlags, LPPROPSHEETPAGE ppsp, ULONG FAR * pcpsp);          
	STDMETHODIMP_ (VOID) FreePages(LPPROPSHEETPAGE ppsp, ULONG ulFlags, ULONG cpsp);          

 private:
 	ULONG m_cRef;
	LPUNKNOWN m_pExchExt;
};

//
// message event extension class
//
class CDGExtMsgEvents : public IExchExtMessageEvents
{
 public:
    CDGExtMsgEvents (LPUNKNOWN pParentInterface) { 
    m_pExchExt = pParentInterface;
    m_cRef = 0; 
	m_bInSubmitState = FALSE;
    };

    STDMETHODIMP QueryInterface(REFIID riid, LPVOID * ppvObj);
    inline STDMETHODIMP_(ULONG) AddRef() { ++m_cRef; return m_cRef; };
    inline STDMETHODIMP_(ULONG) Release() 
	           { ULONG ulCount = --m_cRef;
                   if (!ulCount) { delete this; }
                return ulCount;};

    STDMETHODIMP OnRead(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnReadComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnWrite(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnWriteComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnSubmit(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP_ (VOID)OnSubmitComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);
    STDMETHODIMP OnCheckNames(LPEXCHEXTCALLBACK lpeecb);
    STDMETHODIMP OnCheckNamesComplete(LPEXCHEXTCALLBACK lpeecb, ULONG ulFlags);


 private:
 	ULONG   m_cRef;
	HRESULT m_hrOnReadComplete;
    BOOL    m_bInSubmitState;
	LPUNKNOWN m_pExchExt;

};


#endif // __CDGEXT32_H__
