// ******************************************************************
// SIMPLE OUTBOUND TAPI DIALER APPLICATION
// ******************************************************************
//
// Title:	TAPIOut
// Version:	1.0 - 05/24/96 (MCA)
//
// Equip:	VC++ 4.0 / Win95 / TAPI SDK
//
// Client:	MAPI, SAPI, TAPI Developer's Guide (SAMS 1996)
// 
// Desc:	Simple dialog to show how to use TAPI to place outbound
//			calls. Takes dialing string and shows progress as the
//			program attempts to complete the call.
//
// Files:	TAPIOUT.C
//			TAPIOUT.RC
//			TAPIOUT.DEF
//			RESOURCE.H
//			TAPI.H
//			TAPI32.LIB
//
// ******************************************************************	

// ****************************************
// Includes and defines
//
#include "windows.h"
#include "tapi.h"
#include "resource.h"

#define tapiVersionCur                  (MAKELONG(4,1)) // ver 1.4
#define	TAPI_LINE_REPLY					5000
#define TAPI_LINECALLSTATE_CONNECTED	5001
#define TAPI_LINECALLSTATE_IDLE			5002
#define TAPI_LINECALLSTATE_DISCONNECTED	5003
#define TAPI_LINECALLSTATE_BUSY			5004
#define TAPI_LINECALLSTATE_ACCEPTED		5005
#define TAPI_LINECALLSTATE_PROCEEDING	5006
#define TAPI_LINECALLSTATE_OFFERING		5007
#define TAPI_LINECALLSTATE_DIALTONE		5008
#define TAPI_LINECALLSTATE_DIALING		5009

// ******************************************
// global declares
//
LONG PlaceCall( HWND, LPTSTR );
void CALLBACK LineCallBackProc(DWORD hDevice,DWORD dwMessage,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2,DWORD dwParam3);
BOOL WINAPI MainDialog(HWND hDlg, WORD msg, WORD wParam, LONG lParam);
void ShowProgress( HWND hWnd, LPTSTR OutputString );
void SetVarProps( HWND hWnd, DWORD hDevice );

LINECALLPARAMS LineParams;	// need this structure
DWORD lines;				// count of available lines
HINSTANCE hInst;			// this instance of the app
HWND MainWin, ButtonWnd;	// window handles
HLINEAPP LineHandle = NULL;	// tapi line handle

// *******************************************
// The callback to handle TAPI messages
//
// This routine handles all messages generated by TAPI services.
// Most of these messages are ignored here or just passsed on to
// the main dialog for posting to the progress window.
//
void CALLBACK LineCallBackProc(DWORD hDevice,DWORD dwMessage,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2,DWORD dwParam3)
{   
	
    switch (dwMessage) 
        { 
        case LINE_LINEDEVSTATE:	// we'll ignore these for now...
            switch (dwParam1)
            {
            	case LINEDEVSTATE_REINIT:
            		break;

            	case LINEDEVSTATE_RINGING:
            		break;
            } 
            break;
        
        case LINE_CALLSTATE:	// review the call state messages
            {
				switch( dwParam1 )
				{
					case LINECALLSTATE_IDLE:	// went idle
					{
						LONG retcode;
						LINECALLINFO LineCallInfo;
						//
						// load call info into structure
						memset( &LineCallInfo, 0, sizeof( LINECALLINFO ) );
						LineCallInfo.dwTotalSize = sizeof( LINECALLINFO );
						lineGetCallInfo( (HCALL)hDevice, &LineCallInfo );
						//
						// deallocate the call
						retcode = lineDeallocateCall( (HCALL)hDevice );
						//
						// post message to main dialog
						PostMessage((HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_IDLE, (LPARAM)(HLINE)LineCallInfo.hLine );
						break;	   
					}													     
					case LINECALLSTATE_ACCEPTED:	// just pass message on...
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_ACCEPTED,(LPARAM)(HCALL)hDevice );
					 	break;
					}

					case LINECALLSTATE_PROCEEDING:	// post progress message
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_PROCEEDING,(LPARAM)(HCALL)hDevice );
						break;
					}

					case LINECALLSTATE_CONNECTED:	// hey, we got through!
					{
						//
						// local vars for processing
						LPVARSTRING lpVarStringStruct = NULL;
						size_t sizeofVarStringStruct = sizeof( VARSTRING ) + 1024;
						HANDLE CommFile = NULL;
						long lreturn;

						// get the comm handle.  Be sure to drop this handle when
						// the call is done or you'll get device unavailable errors
						// and have to REBOOT!
						lpVarStringStruct = LocalAlloc( 0, sizeofVarStringStruct );
						do
						{
							memset( lpVarStringStruct, 0, sizeofVarStringStruct );
							lpVarStringStruct->dwTotalSize = (DWORD)sizeofVarStringStruct;
							lreturn = lineGetID( 0, 0, (HCALL)hDevice, LINECALLSELECT_CALL, lpVarStringStruct, "comm/datamodem" );
						} while( lreturn != 0 );
						//
						// get comm device handle and save it to properties area
						CommFile = *( (LPHANDLE )( ( LPBYTE )lpVarStringStruct + lpVarStringStruct->dwStringOffset ) );
						SetProp( (HWND)dwInstance, "HCOMM", CommFile );
						SetVarProps( (HWND)dwInstance, hDevice );
						//
						// tell main dialog we got through
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_CONNECTED, (LPARAM)(HANDLE)CommFile );
						LocalFree( lpVarStringStruct ); // drop mem space
						break;
					}

					case LINECALLSTATE_OFFERING:	// pass it on
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_OFFERING, (LPARAM)(HCALL)hDevice );
						break;
					}

					case LINECALLSTATE_DIALTONE:	// pass it on
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_DIALTONE, (LPARAM)(HCALL)hDevice );
						break;
					}

					case LINECALLSTATE_DIALING:		// pas it on...
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_DIALING, (LPARAM)(HCALL)hDevice );
						break;
					}

					case LINECALLSTATE_BUSY:		// pass it on...
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_BUSY, 0 );
						break;
					}

					case LINECALLSTATE_DISCONNECTED:	// pass it on...
					{
						SetVarProps( (HWND)dwInstance, hDevice );
						PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINECALLSTATE_DISCONNECTED, (LPARAM)(HCALL)hDevice );
						break;
					}
				}
            	break;
            }
        case LINE_CLOSE: // the line has been closed!
            {
            break; 
            }
        case LINE_REPLY:	// pass on TAPI_REPLY messages
		{
			PostMessage( (HWND)dwInstance, WM_COMMAND, TAPI_LINE_REPLY, 0 );
            break;
		}

        //
		// other messages that we'll ignore here
		//
        case LINE_REQUEST:
        case LINE_ADDRESSSTATE:
            break;
        case LINE_CALLINFO:
            break;
        case LINE_DEVSPECIFIC:
            break;
        case LINE_DEVSPECIFICFEATURE:
            break;
        case LINE_GATHERDIGITS:
            break;
        case LINE_GENERATE:
            break;
        case LINE_MONITORDIGITS:
            break;
        case LINE_MONITORMEDIA:
            break;
        case LINE_MONITORTONE:
            break;
        } /* switch */ 
        
} /* LineCallBackProc */


// ***********************************************
// This routine places the actual call
//
LONG PlaceCall( HWND hWnd, LPTSTR PhoneNumber )
{
	LONG	retcode;				// local returns
	DWORD	i;						// counter for lines
	DWORD	ApiVersion;				// expected API version
	DWORD	RetApiVersion;			// return version
	LINEEXTENSIONID	ExtensionID;	// struc for API call
	HLINE	hLine;					// local line handle
	HCALL	hCall;					// local call handle

	//
	// make sure you have a phone number
	if( lstrlen( PhoneNumber ) < 1 )
		return( -1 );	

	//
	// Initialize the line, register the callback
	if( LineHandle == NULL )
		retcode = lineInitialize( &LineHandle, hInst, (LINECALLBACK)LineCallBackProc, "TAPI Out", &lines );

	if( retcode < 0 )
		return( retcode );

	//
	// go through all lines to get API and properties
	// if you find one that has the right properties, 
	// jump out and continue to next section of code
	//
	hLine = (HLINE)GetProp( hWnd, "HLINE" );
	if( hLine == NULL )
	{
		for( i=0; i < lines; i++ )
		{
			// Negotiate the API Version for each line
			ApiVersion = tapiVersionCur;
			retcode = lineNegotiateAPIVersion( LineHandle, i, ApiVersion, ApiVersion, &RetApiVersion,
				&ExtensionID );

			retcode = lineOpen( LineHandle, i, &hLine, RetApiVersion, 0, (DWORD)hWnd,
				LINECALLPRIVILEGE_OWNER | LINECALLPRIVILEGE_MONITOR,
				LINEMEDIAMODE_DATAMODEM, NULL );
			if( retcode == 0 )
				break;
		}
		if( retcode != 0 )
			return( -1 );
	}
	//
	// found a good line
	SetProp( hWnd, "HLINE",(HANDLE)(HLINE)hLine );

	//
	// now set of properties of the line for outbound dialing
	memset( &LineParams, 0, sizeof( LINECALLPARAMS ) );
	LineParams.dwTotalSize = sizeof( LINECALLPARAMS );
	LineParams.dwMinRate = 9600;	// setting data rates
	LineParams.dwMaxRate = 9600;	//
	LineParams.dwMediaMode = LINEMEDIAMODE_DATAMODEM;	// doing a data call
	//
	// finally place the call!
	retcode = lineMakeCall( hLine, &hCall, PhoneNumber, 0, &LineParams );
	return( retcode );	// tell'em how it turned out!
}


// ************************************************
// update list box to show TAPI progress
//
void ShowProgress( HWND hDlg, LPTSTR OutputString )
{
	DWORD dwIndex;
	int i;

	dwIndex = SendDlgItemMessage( hDlg, ID_STATUS_LIST, LB_ADDSTRING, 0, (LPARAM)(LPSTR)OutputString );
	if( dwIndex == LB_ERR )		// clear some space for full box
	{
		for( i = 0; i < 10; i++ )
			SendDlgItemMessage( hDlg, ID_STATUS_LIST, LB_DELETESTRING, 0, 0 );
		// now send the message
		dwIndex = SendDlgItemMessage( hDlg, ID_STATUS_LIST, LB_ADDSTRING, 0, (LPARAM)(LPSTR)OutputString );
	}
	SendDlgItemMessage( hDlg, ID_STATUS_LIST, LB_SETCURSEL, (WPARAM)dwIndex, 0 );
	return;
}


// ***************************************************
// get line handle from LINECALLINFO structure
//
void SetVarProps( HWND hWnd, DWORD hDevice )
{
	LINECALLINFO LineCallInfo;

	memset( &LineCallInfo, 0, sizeof( LINECALLINFO ) );
	SetProp( hWnd, "HCALL", (HANDLE)(HCALL)hDevice );
	LineCallInfo.dwTotalSize = sizeof( LINECALLINFO );
	lineGetCallInfo( (HCALL)hDevice, &LineCallInfo );
 	SetProp( hWnd, "HLINE", (HANDLE)(HLINE)LineCallInfo.hLine );
	return;
}

// ********************************************
// Main Dialog to handle user interface
//
BOOL WINAPI MainDialog(HWND hDlg, WORD msg, WORD wParam, LONG lParam)
{
	switch (msg)
	{
		case WM_INITDIALOG:	// when dialgo first starts up
		{
			// Set the necessary properties to null
			SetProp( hDlg, "HCALL", NULL );
			SetProp( hDlg, "HLINE", NULL );
			SetProp( hDlg, "HCOMM", NULL );
			break;
		}

		case WM_COMMAND:	// user pressed a button
		{
			switch( wParam )
			{			
				case IDOK:	// user pressed the EXIT button
				{
					HCALL hCall; // declare some local vars
					HLINE hLine;
					HANDLE hComm;
					//
					// load the values
					hCall = (HCALL)GetProp( hDlg, "HCALL" );
					hLine = (HLINE)GetProp( hDlg, "HLINE" );
					hComm = (HANDLE)GetProp( hDlg, "HCOMM" );
					//
					// if we have a comm handle, close it
					if( hComm != NULL )					
					{
						CloseHandle( hComm );			
						SetProp( hDlg, "HCOMM", NULL );	
					}
					//
					// if we have a call handle, close it
					if( hCall != NULL )					
					{
						lineDrop( hCall, NULL, 0 );		
						SetProp( hDlg, "HCALL", NULL );	
					}
					//
					// if we have a line handle, close it
					if( hLine != NULL )					
					{
						lineClose( hLine );				
						SetProp( hDlg, "HLINE", NULL );	
					}
					// 
					// close down open line
					if( LineHandle != NULL )
					{
						lineShutdown( LineHandle );
						LineHandle = NULL;		
					}
					//
					// drop the save properties
					RemoveProp( hDlg, "HCALL" );
					RemoveProp( hDlg, "HLINE" );
					RemoveProp( hDlg, "HCOMM" );
					//
					// close down the dialog
	   				EndDialog( hDlg, FALSE );
	   				break;
   				}
   				
   				case ID_CALL:	// user press PLACE CALL
   				{
					char PhoneNumber[ 100 ];	// save some space
					HCALL hCall;				// declare a local handle
					//
					// Gotta call going? - Uh, oh!
					hCall = (HCALL)GetProp( hDlg, "HCALL" );
					if( hCall != NULL  )
					{
					 	MessageBox( hDlg, "Please Disconnect before making another call!", " Tapi Error", MB_ICONSTOP );
						break;
					}
					//
					// Get digits from input box
					GetDlgItemText( hDlg, ID_PHONE, PhoneNumber, sizeof( PhoneNumber ) );
					//
					// place the call (check return value)
					if( PlaceCall( hDlg, PhoneNumber ) < 0 )
						ShowProgress( hDlg, "Unable to start a TAPI Function" );
	   				break;
	   			}

				case ID_DISCONNECT:	// user press DISCONNECT
				{
					LONG retcode;	// some local stuff
					HCALL hCall;
					HANDLE hComm;
					//
					// try to get the handles	
					hCall = (HCALL)GetProp( hDlg, "HCALL" );
					hComm = (HANDLE)GetProp( hDlg, "HCOMM" );
					//
					// if we have a comm handle, drop it	
					if( hComm != NULL )
					{
						CloseHandle( hComm );
						SetProp( hDlg, "HCALL", NULL );
					}
					//
					// if we have a call handle, drop it
					if( hCall != NULL )
					{
						retcode = lineDrop( hCall, NULL, 0 );
						ShowProgress( hDlg, "Call is Dropped" );
						SetProp( hDlg, "HCALL", NULL );
					}
					break;
				}
				//
				// **************************************
				// respond to TAPI Messages
				// **************************************
				//
				case TAPI_LINE_REPLY:
				{
					ShowProgress( hDlg, "Line Reply" ); 
				 	break;
				}
				//
				case TAPI_LINECALLSTATE_CONNECTED:
				{
					ShowProgress( hDlg, "Line Call State is Connected" ); 
					break;
				}						   
				//
				case TAPI_LINECALLSTATE_IDLE:
				{
					LONG retcode;	// local stuff
					HLINE hLine;
					//
					// call went idle, do cleanup
					hLine = (HLINE)GetProp( hDlg, "HLINE" ); 
					//
					// if we have a live line, close it
					if( hLine != NULL )
					{
						retcode = lineClose( hLine );
						SetProp( hDlg, "HLINE", (HANDLE)NULL );
					}
					ShowProgress( hDlg, "Line Call State is idle" );
					break;
				}
				//
				// *********************************************
				// respond to forwarded TAPI messages
				// *********************************************
				//
				case TAPI_LINECALLSTATE_DISCONNECTED:
				{
					ShowProgress( hDlg, "Line Call State is Disconnected" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_BUSY:
				{
					ShowProgress( hDlg, "Line Call State is Busy" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_ACCEPTED:
				{
					ShowProgress( hDlg, "Line Call State is Accepted" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_PROCEEDING:
				{
					ShowProgress( hDlg, "Line Call State is Proceeding" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_OFFERING:
				{
					ShowProgress( hDlg, "Line Call State is Offering" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_DIALTONE:
				{
					ShowProgress( hDlg, "Line Call State is DialTone" );
					break;
				}
				//
				case TAPI_LINECALLSTATE_DIALING:
				{
					ShowProgress( hDlg, "Line Call State is Dialing" );
					break;
				}
				default:
   					break;
   			} // switch (wParam)

			break;
		} // case WM_COMMAND

		default:
			break;
	} // switch (msg)

	return (FALSE);
} // main dialog


// *******************************************
// Initial Entry
//
// establish a message queue
// get the instance handle
// start the user dialog
//
int PASCAL WinMain( HANDLE hInstance, HANDLE hPrev, LPSTR lpCmd, int nShow )
{
   	SetMessageQueue( 100 );
   	hInst = hInstance;
	DialogBox( hInstance, MAKEINTRESOURCE( ID_MAIN_SCREEN ), NULL, MainDialog ); 
   	return( FALSE );
}
  
// ********************************************************************
// END OF FILE
// ********************************************************************