Attribute VB_Name = "LibTAPIPhoneDec"
Option Explicit

' ************************************************
' These structures and constants are needed to
' handle calls to TAPI for phone devices.
' Phone devices are not yet supported by TAPILine
' but may be in the future
' ************************************************

'
' general phone device stuff
'
Global Const PHONE_BUTTON = 14&
Global Const PHONE_CLOSE = 15&
Global Const PHONE_DEVSPECIFIC = 16&
Global Const PHONE_REPLY = 17&
Global Const PHONE_STATE = 18&
'
Global Const PHONEBUTTONFUNCTION_UNKNOWN = &H0&
Global Const PHONEBUTTONFUNCTION_CONFERENCE = &H1&
Global Const PHONEBUTTONFUNCTION_TRANSFER = &H2&
Global Const PHONEBUTTONFUNCTION_DROP = &H3&
Global Const PHONEBUTTONFUNCTION_HOLD = &H4&
Global Const PHONEBUTTONFUNCTION_RECALL = &H5&
Global Const PHONEBUTTONFUNCTION_DISCONNECT = &H6&
Global Const PHONEBUTTONFUNCTION_CONNECT = &H7&
Global Const PHONEBUTTONFUNCTION_MSGWAITON = &H8&
Global Const PHONEBUTTONFUNCTION_MSGWAITOFF = &H9&
Global Const PHONEBUTTONFUNCTION_SELECTRING = &HA&
Global Const PHONEBUTTONFUNCTION_ABBREVDIAL = &HB&
Global Const PHONEBUTTONFUNCTION_FORWARD = &HC&
Global Const PHONEBUTTONFUNCTION_PICKUP = &HD&
Global Const PHONEBUTTONFUNCTION_RINGAGAIN = &HE&
Global Const PHONEBUTTONFUNCTION_PARK = &HF&
Global Const PHONEBUTTONFUNCTION_REJECT = &H10&
Global Const PHONEBUTTONFUNCTION_REDIRECT = &H11&
Global Const PHONEBUTTONFUNCTION_MUTE = &H12&
Global Const PHONEBUTTONFUNCTION_VOLUMEUP = &H13&
Global Const PHONEBUTTONFUNCTION_VOLUMEDOWN = &H14&
Global Const PHONEBUTTONFUNCTION_SPEAKERON = &H15&
Global Const PHONEBUTTONFUNCTION_SPEAKEROFF = &H16&
Global Const PHONEBUTTONFUNCTION_FLASH = &H17&
Global Const PHONEBUTTONFUNCTION_DATAON = &H18&
Global Const PHONEBUTTONFUNCTION_DATAOFF = &H19&
Global Const PHONEBUTTONFUNCTION_DONOTDISTURB = &H1A&
Global Const PHONEBUTTONFUNCTION_INTERCOM = &H1B&
Global Const PHONEBUTTONFUNCTION_BRIDGEDAPP = &H1C&
Global Const PHONEBUTTONFUNCTION_BUSY = &H1D&
Global Const PHONEBUTTONFUNCTION_CALLAPP = &H1E&
Global Const PHONEBUTTONFUNCTION_DATETIME = &H1F&
Global Const PHONEBUTTONFUNCTION_DIRECTORY = &H20&
Global Const PHONEBUTTONFUNCTION_COVER = &H21&
Global Const PHONEBUTTONFUNCTION_CALLID = &H22&
Global Const PHONEBUTTONFUNCTION_LASTNUM = &H23&
Global Const PHONEBUTTONFUNCTION_NIGHTSRV = &H24&
Global Const PHONEBUTTONFUNCTION_SENDCALLS = &H25&
Global Const PHONEBUTTONFUNCTION_MSGINDICATOR = &H26&
Global Const PHONEBUTTONFUNCTION_REPDIAL = &H27&
Global Const PHONEBUTTONFUNCTION_SETREPDIAL = &H28&
Global Const PHONEBUTTONFUNCTION_SYSTEMSPEED = &H29&
Global Const PHONEBUTTONFUNCTION_STATIONSPEED = &H2A&
Global Const PHONEBUTTONFUNCTION_CAMPON = &H2B&
Global Const PHONEBUTTONFUNCTION_SAVEREPEAT = &H2C&
Global Const PHONEBUTTONFUNCTION_QUEUECALL = &H2D&
Global Const PHONEBUTTONFUNCTION_NONE = &H2E&

Type PHONEBUTTONINFO
    dwTotalSize As Long
    dwNeededSize As Long
    dwUsedSize As Long

    dwButtonMode As Long
    dwButtonFunction As Long

    dwButtonTextSize As Long
    dwButtonTextOffset As Long

    dwDevSpecificSize As Long
    dwDevSpecificOffset As Long
End Type
Global Const PHONEBUTTONINFO_FIXEDSIZE = 36

' Note: the "_STR" Types are used to convert from data returned in variable-length strings
' to the fixed structure using LSET

Type PHONEBUTTONINFO_STR
    Mem As String * PHONEBUTTONINFO_FIXEDSIZE
End Type

Global Const PHONEBUTTONMODE_DUMMY = &H1&
Global Const PHONEBUTTONMODE_CALL = &H2&
Global Const PHONEBUTTONMODE_FEATURE = &H4&
Global Const PHONEBUTTONMODE_KEYPAD = &H8&
Global Const PHONEBUTTONMODE_LOCAL = &H10&
Global Const PHONEBUTTONMODE_DISPLAY = &H20&

Global Const PHONEBUTTONSTATE_UP = &H1&
Global Const PHONEBUTTONSTATE_DOWN = &H2&

Type PHONEEXTENSIONID
    dwExtensionID0 As Long
    dwExtensionID1 As Long
    dwExtensionID2 As Long
    dwExtensionID3 As Long
End Type
Global Const PHONEEXTENSIONID_FIXEDSIZE = 16

Type PHONEEXTENSIONID_STR
    Mem As String * PHONEEXTENSIONID_FIXEDSIZE
End Type

Type PHONECAPS
    dwTotalSize As Long
    dwNeededSize As Long
    dwUsedSize As Long

    dwProviderInfoSize As Long
    dwProviderInfoOffset As Long

    dwPhoneInfoSize As Long
    dwPhoneInfoOffset As Long

    dwPermanentPhoneID As Long
    dwPhoneNameSize As Long
    dwPhoneNameOffset As Long
    dwStringFormat As Long

    dwPhoneStates As Long
    dwHookSwitchDevs As Long
    dwHandsetHookSwitchModes As Long
    dwSpeakerHookSwitchModes As Long
    dwHeadsetHookSwitchModes As Long

    dwVolumeFlags As Long
    dwGainFlags As Long
    dwDisplayNumRows As Long
    dwDisplayNumColumns As Long
    dwNumRingModes As Long
    dwNumButtonLamps As Long

    dwButtonModesSize As Long
    dwButtonModesOffset As Long

    dwButtonFunctionsSize As Long
    dwButtonFunctionsOffset As Long

    dwLampModesSize As Long
    dwLampModesOffset As Long

    dwNumSetData As Long
    dwSetDataSize As Long
    dwSetDataOffset As Long

    dwNumGetData As Long
    dwGetDataSize As Long
    dwGetDataOffset As Long

    dwDevSpecificSize As Long
    dwDevSpecificOffset As Long
End Type
Global Const PHONECAPS_FIXEDSIZE = 144

Type PHONECAPS_STR
    Mem As String * PHONECAPS_FIXEDSIZE
End Type

Global Const PHONEERR_ALLOCATED = &H90000001
Global Const PHONEERR_BADDEVICEID = &H90000002
Global Const PHONEERR_INCOMPATIBLEAPIVERSION = &H90000003
Global Const PHONEERR_INCOMPATIBLEEXTVERSION = &H90000004
Global Const PHONEERR_INIFILECORRUPT = &H90000005
Global Const PHONEERR_INUSE = &H90000006
Global Const PHONEERR_INVALAPPHANDLE = &H90000007
Global Const PHONEERR_INVALAPPNAME = &H90000008
Global Const PHONEERR_INVALBUTTONLAMPID = &H90000009
Global Const PHONEERR_INVALBUTTONMODE = &H9000000A
Global Const PHONEERR_INVALBUTTONSTATE = &H9000000B
Global Const PHONEERR_INVALDATAID = &H9000000C
Global Const PHONEERR_INVALDEVICECLASS = &H9000000D
Global Const PHONEERR_INVALEXTVERSION = &H9000000E
Global Const PHONEERR_INVALHOOKSWITCHDEV = &H9000000F
Global Const PHONEERR_INVALHOOKSWITCHMODE = &H90000010
Global Const PHONEERR_INVALLAMPMODE = &H90000011
Global Const PHONEERR_INVALPARAM = &H90000012
Global Const PHONEERR_INVALPHONEHANDLE = &H90000013
Global Const PHONEERR_INVALPHONESTATE = &H90000014
Global Const PHONEERR_INVALPOINTER = &H90000015
Global Const PHONEERR_INVALPRIVILEGE = &H90000016
Global Const PHONEERR_INVALRINGMODE = &H90000017
Global Const PHONEERR_NODEVICE = &H90000018
Global Const PHONEERR_NODRIVER = &H90000019
Global Const PHONEERR_NOMEM = &H9000001A
Global Const PHONEERR_NOTOWNER = &H9000001B
Global Const PHONEERR_OPERATIONFAILED = &H9000001C
Global Const PHONEERR_OPERATIONUNAVAIL = &H9000001D
Global Const PHONEERR_RESOURCEUNAVAIL = &H9000001F
Global Const PHONEERR_REQUESTOVERRUN = &H90000020
Global Const PHONEERR_STRUCTURETOOSMALL = &H90000021
Global Const PHONEERR_UNINITIALIZED = &H90000022
Global Const PHONEERR_REINIT = &H90000023

Global Const PHONEHOOKSWITCHDEV_HANDSET = &H1&
Global Const PHONEHOOKSWITCHDEV_SPEAKER = &H2&
Global Const PHONEHOOKSWITCHDEV_HEADSET = &H4&

Global Const PHONEHOOKSWITCHMODE_ONHOOK = &H1&
Global Const PHONEHOOKSWITCHMODE_MIC = &H2&
Global Const PHONEHOOKSWITCHMODE_SPEAKER = &H4&
Global Const PHONEHOOKSWITCHMODE_MICSPEAKER = &H8&
Global Const PHONEHOOKSWITCHMODE_UNKNOWN = &H10&

Global Const PHONELAMPMODE_DUMMY = &H1&
Global Const PHONELAMPMODE_OFF = &H2&
Global Const PHONELAMPMODE_STEADY = &H4&
Global Const PHONELAMPMODE_WINK = &H8&
Global Const PHONELAMPMODE_FLASH = &H10&
Global Const PHONELAMPMODE_FLUTTER = &H20&
Global Const PHONELAMPMODE_BROKENFLUTTER = &H40&
Global Const PHONELAMPMODE_UNKNOWN = &H80&

Global Const PHONEPRIVILEGE_MONITOR = &H1&
Global Const PHONEPRIVILEGE_OWNER = &H2&

Global Const PHONESTATE_OTHER = &H1&
Global Const PHONESTATE_CONNECTED = &H2&
Global Const PHONESTATE_DISCONNECTED = &H4&
Global Const PHONESTATE_OWNER = &H8&
Global Const PHONESTATE_MONITORS = &H10&
Global Const PHONESTATE_DISPLAY = &H20&
Global Const PHONESTATE_LAMP = &H40&
Global Const PHONESTATE_RINGMODE = &H80&
Global Const PHONESTATE_RINGVOLUME = &H100&
Global Const PHONESTATE_HANDSETHOOKSWITCH = &H200&
Global Const PHONESTATE_HANDSETVOLUME = &H400&
Global Const PHONESTATE_HANDSETGAIN = &H800&
Global Const PHONESTATE_SPEAKERHOOKSWITCH = &H1000&
Global Const PHONESTATE_SPEAKERVOLUME = &H2000&
Global Const PHONESTATE_SPEAKERGAIN = &H4000&
Global Const PHONESTATE_HEADSETHOOKSWITCH = &H8000&
Global Const PHONESTATE_HEADSETVOLUME = &H10000
Global Const PHONESTATE_HEADSETGAIN = &H20000
Global Const PHONESTATE_SUSPEND = &H40000
Global Const PHONESTATE_RESUME = &H80000
Global Const PHONESTATE_DEVSPECIFIC = &H100000
Global Const PHONESTATE_REINIT = &H200000

Type PHONESTATUS
    dwTotalSize As Long
    dwNeededSize As Long
    dwUsedSize As Long

    dwStatusFlags As Long
    dwNumOwners As Long
    dwNumMonitors As Long
    dwRingMode As Long
    dwRingVolume As Long

    dwHandsetHookSwitchMode As Long
    dwHandsetVolume As Long
    dwHandsetGain As Long

    dwSpeakerHookSwitchMode As Long
    dwSpeakerVolume As Long
    dwSpeakerGain As Long

    dwHeadsetHookSwitchMode As Long
    dwHeadsetVolume As Long
    dwHeadsetGain As Long

    dwDisplaySize As Long
    dwDisplayOffset As Long

    dwLampModesSize As Long
    dwLampModesOffset As Long

    dwOwnerNameSize As Long
    dwOwnerNameOffset As Long

    dwDevSpecificSize As Long
    dwDevSpecificOffset As Long
End Type
Global Const PHONESTATUS_FIXEDSIZE = 100

Type PHONESTATUS_STR
    Mem As String * PHONESTATUS_FIXEDSIZE
End Type

Global Const PHONESTATUSFLAGS_CONNECTED = &H1&
Global Const PHONESTATUSFLAGS_SUSPENDED = &H2&

