Attribute VB_Name = "LibTAPIWrapper"
Option Explicit
'
Private udtLineDevCaps() As LINEDEVCAPS
Private cLineDevCapsExtra() As String * 2048
'
Public gDialString As String ' phone number to dial
Public gPlaceCall As Boolean ' ok to place call
Public gName As String ' name to call
Public gLineDev As Integer ' selected line device
'
Public gMinimize As Integer ' minimize at start
Public gMonitor As Integer ' monitor at startup
Public gOutLog As Integer ' log al outbound calls
Public gOrigNumber As String ' for calledID
Public gStartPage As Integer ' for default start page
'
Type DialParams
    DeviceNumber As Integer
    DialableString As String
    Privilege As Long
    MediaMode As Long
End Type
    

Public Function InitTAPI(ctrl As Control, cAppName As String) As Long
    '
    ' perform initial startup of TAPI services
    '
    Dim lRtn As Long
    Dim iNumDev As Long
    Dim iLoop As Integer
    '
    TAPIClearHandles ctrl ' keep it clean!
    '
    ' start it up
    lRtn = ctrl.LineInitialize(cAppName)
    If lRtn < 0 Then
        GoTo TAPIErr
    End If
    '
    ' confirm devices
    iNumDev = ctrl.NumDevices
    For iLoop = 0 To iNumDev - 1
        lRtn = ctrl.LineNegotiateAPIVersion(iLoop, 65536, 65540)
        If lRtn < 0 Then
            GoTo TAPIErr
        End If
    Next
    '
    ' fill line device capabilities structures
    ReDim Preserve udtLineDevCaps(iNumDev)
    ReDim Preserve cLineDevCapsExtra(iNumDev)
    '
    For iLoop = 0 To iNumDev - 1
        lRtn = ctrl.LineGetDevCaps(iLoop)
        If lRtn = 0 Then
            LineDevCapsFunc TAPI_READ, udtLineDevCaps(iLoop), cLineDevCapsExtra(iLoop)
            cLineDevCapsExtra(iLoop) = Clean(cLineDevCapsExtra(iLoop))
        End If
    Next
    '
    GoTo TAPIExit
    '
TAPIErr:
    MsgBox "Unable to Start TAPI Services", vbCritical, "InitTAPI"
    End
    '
TAPIExit:
    InitTAPI = lRtn
    '
End Function

Public Function TAPIDial(ctrl As Control, dpType As DialParams) As Long
    '
    ' dial a selected number
    '
    Dim lRtn As Long
    '
    lRtn = ctrl.LineOpen(dpType.DeviceNumber, 0, dpType.Privilege, dpType.MediaMode)
    If lRtn < 0 Then
        TAPIDial = lRtn
        Exit Function
    End If
    '
    SetCallParams dpType.MediaMode ' load defaults
    '
    ' place the call
    lRtn = ctrl.LineMakeCall(dpType.DialableString, 1)
    '
    TAPIDial = lRtn
    '
End Function

Public Function ReadLineDevCaps(iDevNum As Integer, udtLineDev As LINEDEVCAPS, cExtra As String) As Long
    '
    ' return line caps
    '
    On Error GoTo LocalErr
    '
    Dim lRtn As Long
    '
    lRtn = 0
    udtLineDev = udtLineDevCaps(iDevNum)
    cExtra = Trim(cLineDevCapsExtra(iDevNum))
    '
    Exit Function
    '
LocalErr:
    lRtn = Err
    '
End Function

Public Sub TAPIShutdown(ctrl As Control)
    '
    ' close down TAPI services
    '
    ctrl.LineShutdown
    '
End Sub

Public Sub SetCallParams(lMode As Long)
    '
    ' set up call params data
    '
    Dim cp As LINECALLPARAMS
    Dim cpx As String
    '
    ' defaults for voice calls
    cp.dwBearerMode = LINEBEARERMODE_VOICE
    cp.dwMinRate = 9.6
    cp.dwMaxRate = 28.8
    cp.dwMediaMode = lMode
    cp.dwCallParamFlags = 0
    cp.dwAddressMode = lMode
    cp.DialParams.dwDialPause = 0
    cp.DialParams.dwDialSpeed = 0
    cp.DialParams.dwDigitDuration = 0
    cp.DialParams.dwWaitForDialtone = 0
    cp.dwOrigAddressSize = Len(Trim(gOrigNumber))
    cp.dwOrigAddressOffset = 0
    cp.dwCalledPartySize = 0
    cp.dwCommentSize = 0
    cp.dwUserUserInfoSize = 0
    cp.dwHighLevelCompSize = 0
    cp.dwLowLevelCompSize = 0
    cp.dwDevSpecificSize = 0
    '
    cpx = Trim(gOrigNumber)
    cp.dwTotalSize = Len(cp) + Len(cpx)
    LineCallParamsFunc TAPI_WRITE, cp, cpx
    '
End Sub

Public Sub TAPIHangUp(ctrl As Control)
    '
    ' hang up any current call
    '
    ctrl.LineDrop "", 0
    ctrl.LineDeallocateCall
    ctrl.LineClose
    '
End Sub

Public Sub TAPIClearHandles(ctrl As Control)
    '
    ' clean up handles to a call
    '
    ctrl.HandleToCall = 0
    ctrl.LineHandle = 0
    ctrl.AddressID = 0
    '
End Sub

Public Sub TAPIPlaceCall(ctrl As Control, cdialtarget As String)
    '
    Dim lRtn As Long
    Dim cMsg As String
    Dim dpValue As DialParams
    '
    dpValue.DeviceNumber = gLineDev
    dpValue.Privilege = LINECALLPRIVILEGE_NONE
    dpValue.MediaMode = LINEMEDIAMODE_INTERACTIVEVOICE
    dpValue.DialableString = cdialtarget ' number to call
    '
    lRtn = TAPIDial(ctrl, dpValue)
    If lRtn < 0 Then
        cMsg = "TAPIDial - " & LineErr(lRtn)
        GoTo LocalErr
    End If
    '
    Exit Sub
    '
LocalErr:
    If Err <> 0 Then
        lRtn = Err
        cMsg = Error$
    End If
    '
    MsgBox cMsg, vbCritical, "Dialing Err[" & Hex(lRtn) & "]"
    '
End Sub
